@@ -9,10 +9,10 @@ from pathlib import Path
 from pathlib import PurePath
 import re
 import base64
-from io import StringIO
+from io import StringIO, IOBase
 
 logger = logging.getLogger(__name__)
-logger.setLevel(logging.DEBUG)
+logger.setLevel(logging.INFO)
 
 VALID_STRUCTURE_SEARCH_TYPES = {"substructure", "duplicate",
                         "duplicate_tautomer", "duplicate_no_tautomer",
@@ -233,6 +233,9 @@ class client():
         self.url = creds['url']
         self.session = self.getSession()
 
+    def close(self):
+        self.session.close()
+
     def getSession(self):
         data = {
             'username': self.username,
@@ -319,6 +322,12 @@ class client():
                 filesToUpload[str(file)] = file.open('rb')
         resp = self.session.post("{}/uploads".format(self.url),
                                  files=filesToUpload)
+        # Close the open files
+        for file in filesToUpload:
+            # Check if the file is a file object
+            if isinstance(filesToUpload[file], IOBase):
+                filesToUpload[file].close()
+
         resp.raise_for_status()
         return resp.json()
 
@@ -661,18 +670,24 @@ class client():
             resp.raise_for_status()
         return resp.json()
 
-    def get_experiment_by_code(self, experiment_code):
+    def get_experiment_by_code(self, experiment_code, full = False):
         """Get an experiment from an experiment code
 
         Get an experiment given an experiment code
 
         Args:
             experiment_code (str): An experiment code code
+            full (bool): If true, return the full experiment object
 
         Returns: Returns an experiment object
         """
+
+        params = {}
+        if full:
+            params = {**params, 'fullObject': True}
         resp = self.session.get("{}/api/experiments/codename/{}".
-                                format(self.url, experiment_code))
+                                format(self.url, experiment_code),
+                                params = params)
         if resp.status_code == 500:
             return None
         else:
@@ -772,8 +787,34 @@ class client():
         resp.raise_for_status()
         return resp.json()
 
+    def _dose_response_fit_request(self, dose_response_request_dict):
+        """ Send a dose response fit request to ACAS
+        
+        This is a private method that is used to send a dose response fit json request to ACAS. It is not intended to be used directly.
+
+        Args:
+            dose_response_request_dict (dict): A dictionary containing the request parameters for the dose response fit request.
+        """
+    
+        resp = self.session.post("{}/api/doseResponseCurveFit".format(self.url),
+                                 headers={'Content-Type': 'application/json'},
+                                 data=json.dumps(dose_response_request_dict))
+        resp.raise_for_status()
+        return resp.json()
+
     def experiment_loader(self, data_file, user, dry_run, report_file="",
                           images_file=""):
+        """Load an experiment
+        
+        Load an experiment into ACAS.
+
+        Args:
+            data_file (str): A path to an experiment loader formatted file
+            user (str): A username
+            dry_run (bool): If true, then validate but do not load the data into the database
+            report_file (str): A path to a report file (optional)
+            images_file (str): A path to an images file (optional)
+        """
         data_file = self.upload_files([data_file])['files'][0]["name"]
         if report_file and report_file != "":
             report_file = self.upload_files([report_file])['files'][0]["name"]
@@ -787,6 +828,72 @@ class client():
         resp = self.experiment_loader_request(request)
         return resp
 
+    def dose_response_experiment_loader(self, model_fit_type, fit_settings, **kwargs):
+        """Dose response experiment loader
+        
+        Args:
+            model_fit_type (str): The type of model fit to perform
+            fit_settings (dict): The settings for the model fit
+            **kwargs: All required arguments to pass to the experiment loader (e.g. data_file, user, dry_run = True/False)
+
+        Returns:
+            dict: The response from the experiment loader and doseresponse fit request
+             
+            Example:
+
+                {
+                    "experiment_loader_response": experiment_loader_response_resp_dict,
+                    "dose_response_fit_response": dose_response_fit_response_resp_dict
+                }
+    
+        Example:
+            request = {
+                "data_file": data_file_to_upload,
+                "user": "bob",
+                "dry_run": True,
+                "model_fit_type": "4 parameter D-R",
+                "fit_settings": {
+                    "smartMode":True,
+                    "inactiveThresholdMode":True,
+                    "inactiveThreshold":20,
+                    "theoreticalMaxMode":False,
+                    "theoreticalMax":None,
+                    "inverseAgonistMode":False,
+                    "max":{
+                        "limitType":"none"
+                    },
+                    "min":
+                        {
+                            "limitType":"none"
+                    },
+                    "slope":{
+                        "limitType":"none"
+                    },
+                    "baseline":{
+                        "value":0
+                    }
+                }
+            }
+            response = client.\
+                dose_response_experiment_loader(**request)
+        """
+        resp = self.experiment_loader(**kwargs)
+        response = {
+            "experiment_loader_response": resp,
+            "dose_response_fit_response": None
+        }
+        if resp['hasError'] == False and resp['commit'] == True:
+            request = {
+                "experimentCode": resp['results']['experimentCode'],
+                "modelFitType": model_fit_type,
+                "testMode": False,
+                "user": kwargs['user'],
+                "inputParameters": json.dumps(fit_settings)
+            }
+            dose_response_resp = self._dose_response_fit_request(request)
+            response["dose_response_fit_response"] = dose_response_resp
+        return response
+
     def delete_experiment(self, idOrCode):
         """Delete an experiment
 
